/// Main errors definition
//
// Copyright 2020, Jesús Pérez Lorenzo
//
use failure::Fail;
// use diesel::result::DatabaseErrorKind::UniqueViolation;
// use diesel::result::Error::{DatabaseError, NotFound};
use std::fmt;

///`AppError`  Aplication Errors definition ans display 
///
#[derive(Debug)]
pub enum AppError {
/// when toke not valid 
    NoValidToken, 
    NoValidSession,
    SSLModeError,
    RunningModeError,
    UndefinedCollection,
    RecordAlreadyExists,
    RecordNotFound,
    DatabaseError,
    NoDataStorePool,
    NoAppEnvLoaded,
    NoCertsLoaded,
    SqlDeleteError,
    HasherError,
    MailSendError(String),
    MailError,
//    DatabaseError(diesel::result::Error),
    OperationCanceled,
}

#[allow(clippy::pattern_type_mismatch)]
impl fmt::Display for AppError {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        match self {
            AppError::NoValidToken => write!(f, "No valid token found"),
            AppError::NoValidSession => write!(f, "No valid session found"),
            AppError::SSLModeError => write!(f, "SSL Mode error"),
            AppError::RunningModeError => write!(f, "No valid run mode"),
            AppError::UndefinedCollection => write!(f, "Collection undefined"),
            AppError::RecordAlreadyExists => write!(f, "This record violates a unique constraint"),
            AppError::RecordNotFound => write!(f, "This record does not exist"),
            AppError::NoDataStorePool => write!(f, "No data store pool"),
            AppError::NoAppEnvLoaded => write!(f, "Application environment not loaded.\nReview APP_CONFIG_PATH and config.toml content "),
            AppError::NoCertsLoaded => write!(f, "Certifcations not loaded. Review APP_CONFIG_PATH certs_store_path"),
            AppError::SqlDeleteError => write!(f, "Sql Delete error"),
//            AppError::DatabaseError(e) => write!(f, "Database error: {:?}", e),
            AppError::MailSendError(e) => write!(f, "Mail send error: {:?}", e),
            AppError::MailError => write!(f, "Mail error "),
            AppError::DatabaseError => write!(f, "Database error "),
            AppError::HasherError => write!(f, "Hasher error "),
            AppError::OperationCanceled => write!(f, "The running operation was canceled"),
        }
    }
}
/*
/*
impl From<diesel::result::Error> for AppError {
    fn from(e: diesel::result::Error) -> Self {
        match e {
//            DatabaseError(UniqueViolation, _) => AppError::RecordAlreadyExists,
            NotFound => AppError::RecordNotFound,
            _ => AppError::DatabaseError(e),
        }
    }
}
*/


impl From<BlockingError<AppError>> for AppError {
    fn from(e: BlockingError<AppError>) -> Self {
        match e {
            BlockingError::Error(inner) => inner,
            BlockingError::Canceled => AppError::OperationCanceled,
        }
    }
}

#[derive(Debug, Serialize)]
struct ErrorResponse {
    err: String,
}

impl actix_web::ResponseError for AppError {
    fn error_response(&self) -> HttpResponse {
        let err = format!("{}", self);
        let mut builder = match self {
            AppError::RecordAlreadyExists => HttpResponse::BadRequest(),
            AppError::RecordNotFound => HttpResponse::NotFound(),
            _ => HttpResponse::InternalServerError(),
        };
        builder.json(ErrorResponse { err })
    }

    // fn render_response(&self) -> HttpResponse {
    //     self.error_response()
    // }
}


#[derive(Debug, Fail)]
pub enum AppCertificateError {
    #[fail(display = "{} - Reason: {}", 0, 1)]
    BadFile(String, String),
    #[fail(display = "{} - Reason: {}", 0, 1)]
    FileReadError(String, String),
}

#[derive(Debug, Fail)]
pub enum TokenErrors {
    #[fail(display = "{} - Reason: {}", 0, 1)]
    TokenEncodingFailed(String, String),

    #[fail(display = "{} - Reason: {}", 0, 1)]
    TokenDecodingFailed(String, String),

    #[fail(display = "{} - Reason: {}", 0, 1)]
    MissingServerRefreshToken(String, String),

    #[fail(display = "{} - Reason: {}", 0, 1)]
    InvalidServerRefreshToken(String, String),

    #[fail(display = "{} - Reason: {}", 0, 1)]
    InvalidClientAuthenticationToken(String, String),
}

#[derive(Debug, Fail)]
pub enum LoginFailed {
    #[fail(display = "{} - Reason: {}", 0, 1)]
    MissingPassword(String, String),

    #[fail(display = "{} - Reason: {}", 0, 1)]
    InvalidPassword(String, String),

    #[fail(display = "{} - Reason: {}", 0, 1)]
    InvalidTokenOwner(String, String),

    #[fail(display = "{} - Reason: {}", 0, 1)]
    PasswordHashingFailed(String, String),

    #[fail(display = "{} - Reason: {}", 0, 1)]
    PasswordVerificationFailed(String, String),

}
*/