use std::fs; //, io};
use std::fs::OpenOptions;
use std::io::Write;
use std::path::Path;
use std::collections::HashMap;
use tera::Tera;

use anyhow::{anyhow,Result,Context,Error};

use clds::clouds::defs::{ConfigResources,Server};
use clds::pkgs::{Packages};


pub fn trim_newline(s: &mut String) {
  if s.ends_with('\n') {
    s.pop();
    if s.ends_with('\r') {
      s.pop();
    }
  }
}

/// render Tera 
/// 
pub async fn tera_render(tera: &mut Tera, tpl_context: tera::Context, tpl: &str, output_path: &str, append: bool) -> Result<()> {
  let mut all_tpls = vec![("data-template.html",tpl)];
  // match read_path_file(&path, &template_head, "content") {
  //   Ok(tpl_head) =>  all_tpls.push((&template_head,tpl_head)),
  //   Err(_) => {} // ignore if no header
  // }
  tera.add_raw_templates(all_tpls);
  match tera.render("data-template.html", &tpl_context) {
    Ok(mut res) => if append && Path::new(&output_path).exists() {
        let mut file = OpenOptions::new().append(true).open(&output_path)?;
        res = res.replace("&#x2F;", "/");
        //  println!("{}",&res);
        file.write(res.as_bytes())?;
      } else {
        // fs::write(&output_path, res)?;
        let mut file = OpenOptions::new().write(true).truncate(true).open(&output_path)?;
        file.write_all(&res.as_bytes())?;
    },
    Err(e) => {
      println!("Error 'tera_render': {}",e);
      // return Err(anyhow!("Error tera render: {}", e));
    },
  }
  Ok(())
}

/// tpl_data
pub async fn tpl_data_map(config: &HashMap<String, String>, tpl: &str, output_path: &str, append: bool) -> Result<()> {
  let mut tera = Tera::default();
  match tera::Context::from_serialize(&config) {
    Ok(tpl_context) => {
     // tpl_context.contains_key(index);
     // tpl_context.insert(index);
     tera_render(&mut tera, tpl_context, tpl, output_path, append).await.with_context(|| format!("Failed to render 'data_map' {}", &tpl))?;
    },
    Err(e) => return Err(anyhow!("Error tera context 'data_map' serialize: {:?}", e)),
  }
  Ok(())
}

/// tpl_data
pub async fn tpl_data(config: &ConfigResources, tpl: &str, output_path: &str, append: bool) -> Result<()> {
  let mut tera = tera::Tera::default();
  // let template_head = "header.html";
  //let tpl = read_path_file(&path, &template_name, "content")?;
  match tera::Context::from_serialize(config) {
    Ok(tpl_context) => {
     // tpl_context.contains_key(index);
     // tpl_context.insert(index);
     tera_render(&mut tera, tpl_context, tpl, output_path, append).await.with_context(|| format!("Failed to render 'data' {}", &tpl))?;
    },
    Err(e) => return Err(anyhow!("Error tera context 'data' serialize: {:?}", e)),
  }

  // Ok(res)

  // let mut data_hash: HashMap<String, String> = HashMap::new();
  // let mut tpl_context = tera::Context::new();
  // // println!("data: {} template: {}",data_file,tpl_file);
  // match hash_from_data(data_file, &mut tpl_context, &mut data_hash, true) {
  //   Ok(_) => {
  //     match data_templated(&"".to_string(),tpl_file,&mut tpl_context, &data_hash) {
  //       Ok(result) => println!("{}",result),
  //       Err(e) => println!("Error with {} data in {} template: {}", data_file, tpl_file, e),
  //     }
  //   },
  //   Err(e) => println!("Error with {} data in {} template: {}", data_file, tpl_file, e),
  // }
  Ok(())
}

/// tpl_data
pub async fn tpl_data_server(config: &Server, tpl: &str, output_path: &str, append: bool) -> Result<()> {
//  if let Some(servers) = &config.servers {
    let mut tera = tera::Tera::default();
    match tera::Context::from_serialize(&config) {
      Ok(tpl_context) => {
      // tpl_context.contains_key(index);
      // tpl_context.insert(index);
      tera_render(&mut tera, tpl_context, tpl, output_path, append).await.with_context(|| format!("Failed to render 'data_server' {}:", &tpl))?;
      },
      Err(e) => return Err(anyhow!("Error tera context 'data_server' serialize: {:?}", e)),
    }
//  }
  Ok(())
}


/// yaml
pub fn load_yaml_config(path: String) -> Result<ConfigResources> { // }, serde_yaml::Error> {
  // let mut data= Config::default();
  let content=fs::read_to_string(path)?;
  let data: ConfigResources = serde_yaml::from_str(&content)?;
  // println!("{:#?}",data);
  Ok(data)
}
pub fn load_yaml_packages(path: String) -> Result<Packages> { // }, serde_yaml::Error> {
  // let mut data= Config::default();
  let content=fs::read_to_string(path)?;
  let data: Packages = serde_yaml::from_str(&content)?;
  // println!("{:#?}",data);
  Ok(data)
}